-- HitsMode, by Allara


--
--  HitsMode User Interface Code and Graphics Copyright (C) 2005-2007 Jeremy Fuller (Allara)
--      
--  Please do not use the code or graphics for the user interface without permission.
--  Contact me at allara.starmist@gmail.com for more information.
--


-- This system is very nil-safe, except for one assumption. Because of the order of
-- calls, the option system assumes that if a variable exists, it's entire structure
-- has been created. Therefore, it's important you properly initialize the option
-- system before any calls to Get or Set functions.
--
-- The structure of an option variable is as follows:
--
--   HM_Options["Player of Realm"]                (HM_POptions is set to this on init)
--     ["variable name"]
--       ["value"]                the single value of the option (also the value of sliders)
--       ["window"]
--         1..4                                the individual toggles for each chat window
--       ["color"]
--         r, g, b                the color of the option (0-1 range)
--
-- This system relies on the HM_OptionsList object to load default values. If a
-- default is not present, a 0 is initialized into the field.


-- Starts up the options system
function HM_InitializeOptionsSystem()
        if (HM_Options == nil) then
                HM_InitBlankOptions();
        end
        HM_POptions = HM_GetOptionsPlayer();
        HM_CreateOptionSummary();
end


-- Creates a summary of the options currently configured, for development purposes
function HM_CreateOptionSummary()
        local c;
        
        HM_OptionSummary = { };
        for c = 1, HM_NUMPANELS do
                for key, value in pairs(HM_OptionsList[c]) do
                        if (value.var) then
                                table.insert(HM_OptionSummary, value.text.." ["..value.var.."]");
                        end
                end
        end
end


-- Gets a value from the options system
function HM_GetValue(var, num)
        -- This bit forces GetValue to return a specific window value if we are rendering (for multiwindow support)
        if (HM_CoerceChat == 1) then
                num = HM_CoerceChat_CurrentWindow;
        end

        -- Get the value
        if (HM_POptions[var]) then
                if (num) then
                        if (HM_POptions[var].window[num]) then
                                return HM_POptions[var].window[num];
                        else
                                HM_Print("HitsMode GetValue: "..var.." does not contain [window"..num.."].");
                                return 0;
                        end
                else
                        if (HM_POptions[var].value) then
                                return HM_POptions[var].value;
                        else
                                HM_Print("HitsMode GetValue: "..var.." does not contain [value].");
                                return 0;
                        end
                end
        else
                HM_Print("HitsMode GetValue: "..var.." not found.");
                return 0;
        end
end


-- Returns non-nil if the option is present
function HM_IsPresent(var)
        if (HM_POptions[var]) then
                return 1;
        else
                return 0;
        end
end


-- Sets a value in the options system
function HM_SetValue(var, value, num)
        if (HM_POptions[var]) then
                if (num) then
                        HM_POptions[var].window[num] = value;
                else
                        HM_POptions[var].value = value;
                end
        else
                HM_Print("HitsMode SetValue: "..var.." not found.");
        end
end


-- Sets all possible values in an option variable (useful for "single" options, just to be safe)
function HM_SetAllValues(var, value)
        HM_SetValue(var, value);
        HM_SetValue(var, value, 1);
        HM_SetValue(var, value, 2);
        HM_SetValue(var, value, 3);
        HM_SetValue(var, value, 4);
end


-- Gets a color from the options system
function HM_GetColor(var)
        if (HM_POptions[var]) then
                if (HM_POptions[var].color) then
                        return HM_CopyTable(HM_POptions[var].color);
                else
                        HM_Print("HitsMode GetColor: "..var.." does not contain [color].");
                        return nil;
                end
        else
                HM_Print("HitsMode GetColor: "..var.." not found.");
                return nil;
        end
end


-- Sets a color in the options system
function HM_SetColor(var, value)
        if (HM_POptions[var]) then
                HM_POptions[var].color = value;
        else
                HM_Print("HitsMode SetColor: "..var.." not found.");
        end
end


-- Creates the initial blank object
function HM_InitBlankOptions()
        HM_Options = { };
end


-- Returns a pointer to the current player's option set, or creates a default if none present
function HM_GetOptionsPlayer()
        name = UnitName("player").." of "..GetCVar("realmName");

        -- No current options object found, load all defaults
        if (HM_Options[name] == nil) then
                HM_LoadDefaultOptions(name);
                
        -- Upgrading from a version prior to 1.2, which introduced multiwindow support and requires a reset
        elseif (HM_Options[name]["version"] == "1.0" or HM_Options[name]["version"] == "1.1") then
                HM_Print(HM_BCC(.3, .3, 1).."Upgrading from a version prior to 1.2. Reset required.");
                HM_LoadDefaultOptions(name);
                
        -- Different version found, migrate options
        elseif (HM_Options[name]["version"] ~= HM_Version) then
                HM_Print(HM_BCC(.3, .3, 1).."You are using a different version of HitsMode. Migrating options.");
                HM_MigrateOptions(name);
                
        -- Same version, but we're still going to migrate silently in case something got corrupted
        else
                HM_MigrateOptions(name, 1);
        end
        
        return HM_Options[name];
end


-- Makes a deep copy of a given table (gets around LUA's annoying copy-by-reference philosophy)
-- The second argument is for internal use during recursing
function HM_CopyTable(t, lookup_table)
        local copy = {};
        for i,v in pairs(t) do
                if type(v) ~= "table" then
                        copy[i] = v;
                else
                        lookup_table = lookup_table or {};
                        lookup_table[t] = copy;
                        if lookup_table[v] then
                                copy[i] = lookup_table[v]; -- We already copied this table, reuse the copy
                        else
                                copy[i] = HM_CopyTable(v,lookup_table); -- Not yet copied, copy it
                        end
                end
        end
        return copy;
end


function HM_SaveGlobalOptions()
        local name = UnitName("player").." of "..GetCVar("realmName");
        HM_POptions = nil;
        HM_Options["global"] = HM_CopyTable(HM_Options[name]);
        HM_SetInfo("The current options have been saved as the Global options.");
        HM_POptions = HM_Options[name];
end


function HM_LoadGlobalOptions()
        local name = UnitName("player").." of "..GetCVar("realmName");
        HM_POptions = nil;
        if (HM_Options["global"] == nil) then
                HM_SetInfo("No global options have been saved yet! Can't load.");
        else
                HM_Options[name] = HM_CopyTable(HM_Options["global"]);
                HM_SetInfo("The Global options have been loaded.");
        end
        HM_POptions = HM_Options[name];
        HM_UpdateOptions();
end


function HM_LoadDefaultButton()
        local name = UnitName("player").." of "..GetCVar("realmName");
        HM_POptions = nil;
        HM_LoadDefaultOptions(name);
        HM_POptions = HM_Options[name];
        HM_UpdateOptions();
end


-- Creates a new default configuration for the current player
function HM_LoadDefaultOptions(name, silent)
        local c, p;

        HM_Options[name] = nil;
        HM_Options[name] = { };
        p = HM_Options[name];
        p["version"] = HM_Version;
        
        for c = 1, HM_NUMPANELS do
                for key, value in pairs(HM_OptionsList[c]) do
                        if (value.var) then
                                if (p[value.var] == nil) then
                                        p[value.var] = { };
                                end
                                if (value.default) then
                                        p[value.var].value = value.default;
                                else
                                        p[value.var].value = 0;
                                end
                                p[value.var].window = { };
                                if (value.default1) then
                                        p[value.var].window[1] = value.default1;
                                else
                                        p[value.var].window[1] = 0;
                                end
                                if (value.default2) then
                                        p[value.var].window[2] = value.default2;
                                else
                                        p[value.var].window[2] = 0;
                                end
                                if (value.default3) then
                                        p[value.var].window[3] = value.default3;
                                else
                                        p[value.var].window[3] = 0;
                                end
                                if (value.default4) then
                                        p[value.var].window[4] = value.default4;
                                else
                                        p[value.var].window[4] = 0;
                                end
                                if (value.defaultColor) then
                                        p[value.var].color = value.defaultColor;
                                else
                                        p[value.var].color = {
                                                r = 0, g = 0, b = 0
                                        };
                                end
                        end
                end
        end
        
        p["debug"] = {
                value = 0,
                window = {
                        [1] = 0,
                        [2] = 0,
                        [3] = 0,
                        [4] = 0
                }
        };
        
        if (silent == nil) then
                HM_Print(HM_BCC(.3, .3, 1).."Default options loaded for "..name..".");
        end
end


-- Migrates options from an older version. Doesn't do validation on existing values,
-- just finds all missing values and loads defaults in without overwriting existing options
function HM_MigrateOptions(name, silent)
        local c, p, count, oldversion;

        p = HM_Options[name];
        oldversion = p["version"];
        p["version"] = HM_Version;
        count = 0;
        
        for c = 1, HM_NUMPANELS do
                for key, value in pairs(HM_OptionsList[c]) do
                        if (value.var) then
                                if (p[value.var] == nil) then
                                        p[value.var] = { };
                                end
                                if (p[value.var].value == nil) then
                                        if (value.default) then
                                                p[value.var].value = value.default;
                                        else
                                                p[value.var].value = 0;
                                        end
                                        count = count + 1;
                                end
                                if (p[value.var].window == nil) then
                                        p[value.var].window = { };
                                end
                                if (p[value.var].window[1] == nil) then
                                        if (value.default1) then
                                                p[value.var].window[1] = value.default1;
                                        else
                                                p[value.var].window[1] = 0;
                                        end
                                        count = count + 1;
                                end
                                if (p[value.var].window[2] == nil) then
                                        if (value.default2) then
                                                p[value.var].window[2] = value.default2;
                                        else
                                                p[value.var].window[2] = 0;
                                        end
                                        count = count + 1;
                                end
                                if (p[value.var].window[3] == nil) then
                                        if (value.default3) then
                                                p[value.var].window[3] = value.default3;
                                        else
                                                p[value.var].window[3] = 0;
                                        end
                                        count = count + 1;
                                end
                                if (p[value.var].window[4] == nil) then
                                        if (value.default4) then
                                                p[value.var].window[4] = value.default4;
                                        else
                                                p[value.var].window[4] = 0;
                                        end
                                        count = count + 1;
                                end
                                if (p[value.var].color == nil) then
                                        if (value.defaultColor) then
                                                p[value.var].color = value.defaultColor;
                                        else
                                                p[value.var].color = {
                                                        r = 0, g = 0, b = 0
                                                };
                                        end
                                        count = count + 1;
                                end
                        end
                end
        end
        
        if (p["debug"] == nil) then
                p["debug"] = {
                        value = 0,
                        window = {
                                [1] = 0,
                                [2] = 0,
                                [3] = 0,
                                [4] = 0
                        }
                };
        end
        
        if (silent == nil) then
                HM_Print(HM_BCC(.3, .3, 1).."Options migrated for "..name.." from version "..oldversion..".");
                HM_Print(HM_BCC(.3, .3, 1)..count.." new default values loaded.");
        end
end
